'use strict';

/**
 * base logic
 * inherits from base controller
 * @type {Class}
 */
export default class extends think.controller.base {
  /**
   * check auth
   * @return {Promise} []
   */
  // checkAuth(){

  // }
  /**
   * parse validate data
   * {
   *   name: 'required|int|min:10|max:20',
   *   title: 'length:10,20|default:welefen|get',
   *   emai: 'required|email:{}',
   *   ids: 'required|array|int'
   * }
   * @param  {Array}  data []
   * @return {Array}      []
   */
  _parseValidateData(data = {}){
    let result = {};
    for(let name in data){
      let itemData = {};
      
      if(think.isString(data[name])){
        let rules = data[name].split('|');
        rules.forEach(item => {
          item = item.trim();
          if(!item){
            return;
          }
          let pos = item.indexOf(':');
          if(pos > -1){
            let name = item.substr(0, pos);
            let args = item.substr(pos + 1).trim();
            if(args[0] === '{' || args[0] === '['){
              args = [(new Function('', `return ${args}`))()];
            }else if(name !== 'default'){
              args = args.split(/\s*,\s*/);
            }
            itemData[name] = args;
          }else{
            itemData[item] = true;
          }
        });
      }else{
        itemData = data[name];
      }

      let method = this.http.method.toLowerCase();
      if(itemData.get){
        method = 'get';
        delete itemData.get;
      }else if(itemData.file){
        method = 'file';
        delete itemData.file;
      }
      let value = this[method](name);

      if(!value && itemData.default){
        value = itemData.default;
      }
      delete itemData.default;

      if(itemData.int && !itemData.array){
        value = parseInt(value, 10);
      }else if(itemData.float && !itemData.array){
        value = parseFloat(value);
      }else if(itemData.array){
        if(!think.isArray(value)){
          value = think.isString(value) ? value.split(/\s*,\s*/) : [value];
        }
        value = value.map(itemValue => {
          if(itemData.int){
            return parseInt(itemValue, 10);
          }else if(itemData.float){
            return parseFloat(itemValue);
          }
          return itemValue;
        });
      }else if(itemData.boolean){
        if(!think.isBoolean(value)){
          value = ['yes', 'on', '1', 'true'].indexOf(value) > -1;
        }
      }else if(itemData.object){
        if(!think.isObject(value)){
          try{
            value = JSON.parse(value);
          }catch(e){
            value = '';
          }
        }
      }else{
        itemData.string = true;
      }
      //set value to request
      this[method](name, value);
      itemData.value = value;
      result[name] = itemData;
    }
    return result;
  }
  /**
   * validate data
   * this.validate({
   *   welefen: 'required|length:4,20|alpha',
   *   email: 'required|email',
   *   title: 'required|maxLength:10000'
   * })
   * @param  {Object} data      []
   * @return {}           []
   */
  validate(data) {
    this._validateInvoked = true;
    if(think.isEmpty(data)){
      return true;
    }
    data = this._parseValidateData(data);
    let ret = think.validate(data, this.locale());
    if(!think.isEmpty(ret)){
      this.assign('errors', ret);
      return false;
    }
    return true;
  }
  /**
   * get validate errors
   * @return {Object} []
   */
  errors() {
    return this.assign('errors');
  }
  /**
   * auto validate
   * @return {} []
   */
  __after(){
    if(think.isEmpty(this.rules) || this._validateInvoked){
      return;
    }
    let flag = this.validate(this.rules);
    if(!flag){
      let error = this.config('error');
      return this.fail(error.validate_errno, this.errors());
    }
  }
}